package test.saveref;


import java.io.File;
import java.util.Iterator;
import java.util.Properties;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.teneo.hibernate.HbDataStore;
import org.eclipse.emf.teneo.hibernate.HbHelper;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Environment;
import org.hibernate.dialect.HSQLDialect;

public class TestSaveref extends TestCase {
    private static final String DBNAME="TestSaverefDB";
    
    private static Properties getHibernateProperties(String dbName) {
        final Properties props = new Properties(); 
        props.setProperty(Environment.DRIVER, "org.hsqldb.jdbcDriver");
        props.setProperty(Environment.USER, "sa");
        props.setProperty(Environment.URL, "jdbc:hsqldb:file:" + dbName + ";shutdown=true");
        props.setProperty(Environment.PASS, "");
        props.setProperty(Environment.DIALECT, HSQLDialect.class.getName());
        props.setProperty(Environment.CURRENT_SESSION_CONTEXT_CLASS, "thread");
        props.setProperty(Environment.STATEMENT_BATCH_SIZE, "1");
        return props;
    }

    public static void removeDBFiles(String dbname) {
        for (String dbFile: new File(".").list()) {
            File f = new File(dbFile);
            if (f.getName().startsWith(dbname)) {
                if (!f.delete()) {
                    throw new RuntimeException("Lock still held on DB file" + f.getName());
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see junit.framework.TestCase#setUp()
     */
    @Override
    protected void setUp() throws Exception {
        removeDBFiles(DBNAME);
    }

    private HbDataStore createHbDataStore(String dbname) {
        Properties props = getHibernateProperties(dbname);
        EPackage[] packages = { SaverefPackage.eINSTANCE };
        HbDataStore hbds = HbHelper.INSTANCE.createRegisterDataStore(dbname);
        hbds.setEPackages(packages);
        // set the relational database to use and other properties
        hbds.setHibernateProperties(props);
        hbds.setPersistenceProperties(props);
        hbds.initialize();
        return hbds;
    }
    
    public void test() {
        SessionFactory sessionFactory = createHbDataStore(DBNAME).getSessionFactory();
        
        {
            Session session = sessionFactory.openSession();
            session.beginTransaction();

            Root server = SaverefFactory.eINSTANCE.createRoot();
            session.save(server);

            ForeignAuthor author = SaverefFactory.eINSTANCE.createForeignAuthor();
            author.setName("Umberto Eco");
            server.getAuthors().add(author);

            session.save(author);

            ScifiBook book1 = SaverefFactory.eINSTANCE.createScifiBook();
            book1.setName("The Name of the Rose");
            server.getBooks().add(book1);
            

            ScifiBook book2 = SaverefFactory.eINSTANCE.createScifiBook();
            book2.setName("Foucault's Pendulum");
            server.getBooks().add(book2);
            
            book2.getAuthors().add(author);
            book1.getAuthors().add(author);

            session.save(book1);
            session.save(book2);

            System.out.println("book1 author count = " + book1.getAuthors().size());
            System.out.println("book2 author count = " + book2.getAuthors().size());
            
            session.getTransaction().commit();
            session.close();
        }
        
        {
            Session session = sessionFactory.openSession();
            session.beginTransaction();

            Query query = session.createQuery("FROM Book");          
            Iterator iter = query.list().listIterator();
            
            while (iter.hasNext())
            {
                ScifiBook book = (ScifiBook)iter.next();
                Assert.assertEquals("Book \"" + book.getName() + "\" has incorrect number of authors.", 1, book.getAuthors().size());
            }

            session.close();
        }
    }
}
